/// <reference types="node" />
export type PendingPromise<R> = {
    promise: Promise<R>;
    resolve: (value: R | PromiseLike<R>) => void;
    reject: (reason?: any) => void;
};
/**
 * 创建一个待定状态的 {@link Promise}，还有为此 Promise 设置结果的函数。
 */
export declare function createPendingPromise<R extends any = any>(): PendingPromise<R>;
import * as dns from "node:dns";
/**
 * 解析指定域名的 A 记录。
 * @param name 要解析的域名。
 * @param timeout 超时时间，单位为毫秒。
 * @returns 解析结果（一个长度不为0的数组），或者空。
 */
export declare function dnsResolve4(name: string, timeout?: number): Promise<string[] | null>;
/**
 * 解析指定域名的 AAAA 记录。
 * @param name 要解析的域名。
 * @param timeout 超时时间，单位为毫秒。
 * @returns 解析结果（一个长度不为0的数组），或者空。
 */
export declare function dnsResolve6(name: string, timeout?: number): Promise<string[] | null>;
/**
 * 解析指定的域名。
 * @param name 要解析的域名。
 * @param timeout 超时时间，单位为毫秒。
 * @returns 解析结果（一个长度不为0的数组），或者空。
 */
export declare function dnsLookup(name: string, timeout?: number): Promise<dns.LookupAddress[]>;
//# sourceMappingURL=lib.d.ts.map