import { getServerAddressInfo } from "./lib/serverAddr.js";
import { pingJava } from "./lib/java.js";
import { pingBedrock } from "./lib/bedrock.js";
export { mcping };
async function mcping(host, option) {
    if (typeof host !== "string") {
        option = host;
        host = host.serverAddr;
    }
    if (typeof option === "number")
        option = { serverPort: option };
    else if (typeof option === "string")
        option = { serverType: option };
    else if (!option)
        option = {};
    const { serverAddr = host, serverType = "unknown", serverPort, forceHostName, protocolVersionCode = 4, resolveSrvRecord = true, addressFamily, preferIpv6, throwsOnInvalid = true, throwsOnFail = false, serverAddressFilter } = option;
    let reason = [];
    let java;
    let bedrock;
    let addressJava = null;
    let addressBedrock = null;
    if (serverType === "unknown" || serverType === "java") {
        const serverAddressInfoJava = await getServerAddressInfo(host, {
            serverType: "java",
            serverPort,
            resolveSrvRecord,
            family: addressFamily,
            preferIpv6,
            throwsOnInvalid: false
        });
        //@ts-ignore
        addressJava = serverAddressInfoJava;
        if (!serverAddressInfoJava.valid) {
            reason[reason.length] = serverAddressInfoJava.invalidReason;
        }
        let addressIsValid = true;
        if (serverAddressInfoJava.valid && serverAddressFilter != null) {
            try {
                addressIsValid = serverAddressFilter(serverAddressInfoJava.ip, serverAddressInfoJava.port);
                if (!addressIsValid) {
                    reason[reason.length] = new Error("address check fail from serverAddressFilter");
                }
            }
            catch (e) {
                reason[reason.length] = e;
                addressIsValid = false;
            }
        }
        //@ts-ignore
        addressJava.addressIsValid = addressIsValid;
        if (serverAddressInfoJava.valid && addressIsValid) {
            java = await new Promise((resolve, reject) => {
                pingJava({
                    ip: serverAddressInfoJava.ip,
                    port: serverAddressInfoJava.port,
                    serverAddr: forceHostName ?? (serverAddressInfoJava.srvRecord ? serverAddressInfoJava.srvRecord.ip : serverAddr),
                    protocolVersion: protocolVersionCode
                }, (e, r) => {
                    if (e) {
                        if (throwsOnFail && serverType === "java")
                            reject(e);
                        else
                            resolve(undefined);
                        reason[reason.length] = e;
                    }
                    else {
                        resolve(r);
                    }
                }, 5000);
            });
        }
    }
    if (serverType === "unknown" || serverType === "bedrock") {
        const serverAddressInfo = await getServerAddressInfo(host, {
            serverType: "bedrock",
            serverPort,
            resolveSrvRecord,
            family: addressFamily,
            preferIpv6,
            throwsOnInvalid: false
        });
        //@ts-ignore
        addressBedrock = serverAddressInfo;
        if (!serverAddressInfo.valid) {
            reason[reason.length] = serverAddressInfo.invalidReason;
        }
        let addressIsValid = true;
        if (serverAddressInfo.valid && serverAddressFilter != null) {
            try {
                addressIsValid = serverAddressFilter(serverAddressInfo.ip, serverAddressInfo.port);
                if (!addressIsValid) {
                    reason[reason.length] = new Error("address check fail from serverAddressFilter");
                }
            }
            catch (e) {
                reason[reason.length] = e;
                addressIsValid = false;
            }
        }
        //@ts-ignore
        addressBedrock.addressIsValid = addressIsValid;
        if (serverAddressInfo.valid && addressIsValid) {
            bedrock = await new Promise((resolve, reject) => {
                pingBedrock(serverAddressInfo.ip, serverAddressInfo.port, (e, r) => {
                    if (e) {
                        if (throwsOnFail && serverType === "bedrock")
                            reject(e);
                        else
                            resolve(undefined);
                        reason[reason.length] = e;
                    }
                    else {
                        resolve(r);
                    }
                }, 5000);
            });
        }
    }
    let result = {
        status: false
    };
    if (java || bedrock) {
        result.status = true;
    }
    if (bedrock) {
        result.bedrock = bedrock;
    }
    if (java) {
        result.java = java;
    }
    if (addressJava) {
        result.addressJava = addressJava;
    }
    if (addressBedrock) {
        result.addressBedrock = addressBedrock;
    }
    if (!result.status) {
        let finalError;
        if (reason.length > 1) {
            finalError = new AggregateError(reason);
        }
        else if (reason[0] instanceof Error) {
            finalError = reason[0];
        }
        else if (reason[0] != null) {
            finalError = new Error(reason[0]);
        }
        else {
            finalError = new Error("unknown error");
        }
        result.reason = finalError;
    }
    if (throwsOnFail && !result.status) {
        throw result.reason;
    }
    else {
        return result;
    }
}
/*
 //有点抽象，先不用这个
 
class MinecraftBedrockServerPinger extends MinecraftServerPinger<BedrockPingResult> {
   constructor(){
   }
}

class MinecraftJavaServerPinger extends MinecraftServerPinger<JavaPingResult> {
   constructor(){
   }
}

class MinecraftServerPinger<Result> extends NetworkPinger<Result, ServerValidAddressInfo> {
   constructor(){
   }
   public checkAddress(): boolean {
      
   }
}

abstract class NetworkPinger<Result, AddressInfo> {
   protected addressInfo: AddressInfo;
   public pingServer(): Promise<Result> {
      if (!this.checkAddress()){
         return { status: false, addressIsValid: false };
      }
      return { status: false };
   }
   public checkAddress(): boolean {
      return true;
   }
   public constructor(addressInfo: AddressInfo){
      this.addressInfo = AddressInfo;
   }
}

*/ 
//# sourceMappingURL=mcping.js.map